from base64 import b16decode
import time
from datetime import datetime
from random import *
try:
    from rpi_ws281x import *
    import RPi.GPIO as GPIO
    import paho.mqtt.client as mqtt
except ImportError:
    print('Some modules are missing. Try to install them with "pip3 install -r requirements.txt"')
    exit()

# LED strip configuration:
LED_COUNT      = 60      # Number of LED pixels.
LED_PIN        = 18      # GPIO pin connected to the pixels (18 uses PWM!).
#LED_PIN        = 10      # GPIO pin connected to the pixels (10 uses SPI /dev/spidev0.0).
LED_FREQ_HZ    = 800000  # LED signal frequency in hertz (usually 800khz)
LED_DMA        = 10      # DMA channel to use for generating signal (try 10)
LED_BRIGHTNESS = 255     # Set to 0 for darkest and 255 for brightest
LED_INVERT     = False   # True to invert the signal (when using NPN transistor level shift)
LED_CHANNEL    = 0       # set to '1' for GPIOs 13, 19, 41, 45 or 53

global animation_r
animation_r = 255
global animation_g
animation_g = 0
global animation_b
animation_b = 0

def debug_print(message: str):
    now = datetime.now()
    current_time = now.strftime("%H:%M:%S")
    print('[DEBUG]['+current_time+'] '+message)

def wheel(pos):
    if pos < 85:
        return Color(pos * 3, 255 - pos * 3, 0)
    elif pos < 170:
        pos -= 85
        return Color(255 - pos * 3, 0, pos * 3)
    else:
        pos -= 170
        return Color(0, pos * 3, 255 - pos * 3)
 
def rainbowCycle(strip, wait_ms=5, iterations=10):
    for j in range(256*iterations):
        if mode > 0:
            break
        elif power == "False":
            break
        for i in range(strip.numPixels()):
            strip.setPixelColor(i, wheel((int(i * 256 / strip.numPixels()) + j) & 255))
        strip.show()
        time.sleep(wait_ms/1000.0)
    
def setColor(strip, color, wait_ms=10):
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, color)
        strip.show()
        time.sleep(float(0.01))
    time.sleep(wait_ms/1000.0)

def strobe(strip, color):
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, color)
    strip.show()
    time.sleep(float(0.05))
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, Color(0, 0, 0))
    strip.show()
    time.sleep(float(0.05))

def randomColorPerLED(strip):
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, Color(randint(0, 255), randint(0, 255), randint(0, 255)))
    strip.show()
    time.sleep(1)

def randomColor(strip):
    color = Color(randint(0, 255), randint(0, 255), randint(0, 255))
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, color)
    strip.show()
    time.sleep(1)

def colorFadeIn(strip):
    color = Color(randint(0, 255), randint(0, 255), randint(0, 255))
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, color)
        strip.show()
        time.sleep(float(0.01))
    time.sleep(1)

def mqtt_on_connect(client, userdata, flags, rc):
    client.subscribe("server-rack-led/power")
    client.subscribe("server-rack-led/mode")
    client.subscribe("server-rack-led/rgb")
    

if __name__ == '__main__':
    def mqtt_on_message(client, userdata, msg):
        print("topic: " ,str(msg.topic))
        print("payload: " ,str(msg.payload.decode("utf-8")))
        topic = str(msg.topic)
        payload = str(msg.payload.decode("utf-8"))
        if topic == "server-rack-led/power":
            client.publish("server-rack-led/power/status", str(payload))
            global power
            power = payload
        
        elif topic == "server-rack-led/rgb":
            client.publish("server-rack-led/rgb/status", str(payload))
            splitted_payload = payload.split(",")
            global r
            r = str(splitted_payload[0])
            global g
            g = str(splitted_payload[1])
            global b
            b = str(splitted_payload[2])

        elif topic == "server-rack-led/mode":
            client.publish("server-rack-led/mode/status", str(payload))
            if payload == "Rainbow":
                global mode
                mode = 0
            elif payload == "Custom Color":
                mode = 1
            elif payload == "Strobe":
                mode = 2
            elif payload == "Random Color":
                mode = 3
            elif payload == "Random Color per LED":
                mode = 4
            elif payload == "Color fade-in":
                mode = 5

    strip = Adafruit_NeoPixel(LED_COUNT, LED_PIN, LED_FREQ_HZ, LED_DMA, LED_INVERT, LED_BRIGHTNESS, LED_CHANNEL)
    strip.begin()

    mode = 1
    power = "True"
    global r
    r = 255
    global g
    g = 255
    global b
    b = 255

    mqttBroker ="homeassistant.ping-mee.local"
    client = mqtt.Client("server_rack_led")
    client.username_pw_set("mqtt", "pmMQTT_11!")
    debug_print("Connecting to MQTT Broker "+str(mqttBroker))
    try:
        client.connect(mqttBroker)
    except:
        time.sleep(20)
        client.connect(mqttBroker)
    client.on_connect = mqtt_on_connect
    client.on_message = mqtt_on_message

    client.publish("server-rack-led/power/status", "True")
    client.publish("server-rack-led/mode/status", "Custom Color")

    try:
        client.loop_start()
        while True:
            if power == "True":
                client.publish("server-rack-led/power/status", "True")

                if mode == 0:
                    rainbowCycle(strip)
                    client.publish("server-rack-led/mode/status", "Rainbow")
                elif mode == 1:
                    setColor(strip, Color(int(r), int(g), int(b)))
                    client.publish("server-rack-led/mode/status", "Custom Color")
                elif mode == 2:
                    strobe(strip, Color(int(r), int(g), int(b)))
                    client.publish("server-rack-led/mode/status", "Strobe")
                elif mode == 3:
                    randomColor(strip)
                    client.publish("server-rack-led/mode/status", "Random Color")
                elif mode == 4:
                    randomColorPerLED(strip)
                    client.publish("server-rack-led/mode/status", "Random Color per LED")
                elif mode == 5:
                    colorFadeIn(strip)
                    client.publish("server-rack-led/mode/status", "Color fade-in")
            else:
                client.publish("server-rack-led/power/status", "False")
                setColor(strip, Color(0,0,0))

    except KeyboardInterrupt:
        client.publish("server-rack-led/power/status", "False")
        client.loop_stop()
        setColor(strip, Color(0,0,0))
        GPIO.cleanup()
